// Class: GridIterator
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.util.Iterator;

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.Location;

/**
 *  Grid Iterator Lab:<br>
 *
 *    A GridIterator object provides an iterator that that
 *    steps through the locations of a BoundedGrid object 
 *    in a particular order.
 *
 *    <p>
 *    GridIterator implementations need not support the remove
 *    method, as it does not make sense to remove locations
 *    from a bounded grid.
 *
 *  @author Alyce Brady
 *  @version 1 September 2002
 *  @see edu.kzoo.grid.BoundedGrid
 *  @see edu.kzoo.grid.Location
 **/
public abstract class GridIterator implements Iterator
{
    // Encapsulated data for each GridIterator object
    private BoundedGrid grid;     // grid through which to iterate
    private Location nextLoc;     // next location in grid

    /** Constructs an iterator object that steps through a grid
     *  starting at location (0, 0).
     *      @param grid     grid to iterate through
     **/
    public GridIterator(BoundedGrid grid)
    {
        this(grid, new Location(0, 0));
    }

    /** Constructs an iterator object that steps through a grid,
     *  starting at a given location.
     *      @param grid     grid to iterate through
     *      @param startingLoc     where to start the iteration
     **/
    public GridIterator(BoundedGrid grid, Location startingLoc)
    {
        this.grid = grid;
        nextLoc = startingLoc;
    }

    /** Returns grid through which this iterator iterates.
     **/
    public BoundedGrid grid()
    {
        return grid;
    }

    /** Returns true if the grid has more locations.
     **/
    public boolean hasNext()
    {
        // Return true if the next location is valid in the grid.
        return grid().isValid(nextLocation());
    }

    /** Returns next location (pure accessor -- does not advance iterator).
     **/
    protected Location nextLocation()
    {
        return nextLoc;
    }

    /** Returns the next location in the grid.
     **/
    public Object next()
    {
        Location locToReturn = nextLoc;
        nextLoc = findNextLocation();
        return locToReturn;
    }

    /** Removes from the underlying collection the last element
     *  returned by the iterator (not supported).
     **/
    public void remove()
    {
        throw new UnsupportedOperationException("remove method not supported");
    }

    /** Helper Method: Finds next location for iterator.
     **/
    protected abstract Location findNextLocation();

}
