// Class: IteratorLab
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

/**
 *  Grid Iterator Lab:<br>
 *
 *    The IteratorLab class provides a main method that creates an
 *    animation of 2-D structure traversals using iterators.
 *
 *  @author Alyce Brady
 *  @version 1 September 2004
 **/

public class IteratorLab
{
    // Specify dimensions of graphics display for environment.
    private static final int DISPLAY_WIDTH = 400;
    private static final int DISPLAY_HEIGHT = 400;

    // Specify constants used to initialize slider, which adjusts the speed
    // of the program.  The slider controls a timer that inserts a delay or pause
    // after displaying the environment to allow the user time to view it.
    // The longest delay value defines the slowest rate; the shortest delay
    // defines the fastest rate.  Both delay times are in milliseconds.
    private static final int LONGEST_DELAY_MSECS = 1000;  // slowest
    private static final int SHORTEST_DELAY_MSECS = 100;  // fastest

    /**
     *  Start the 2-D Traversal Algorithms program.
     *  The String arguments (args) are not used in this application.
     **/
    public static void main(String[] args)
    {
        // Construct a list of the iterators to use in various traversals.
        IterationController.register("Row-Major Order", RowMajorGridIterator.class);
        IterationController.register("Col-Major Order", ColMajorGridIterator.class);

        // Construct a graphical user interface (GUI).  The GUI
        // provides buttons to run the traversal algorithms and
        // displays the results of each traversal by highlighting
        // the appropriate cells in the environment.
        IteratorGUI gui =
            new IteratorGUI("Traversal Animations", 
                              DISPLAY_WIDTH, DISPLAY_HEIGHT,
                              LONGEST_DELAY_MSECS, SHORTEST_DELAY_MSECS);
    }

}
